/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diff;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Collections;

import fr.aviz.diff.Diff.Operation;
import fr.aviz.diff.Diff.Type;
import fr.aviz.diff.methods.PonsDiff;
import fr.aviz.diffanimation.Document;

public class DiffUtils {

	public static Diff createDiff(Class diffMethod, String textv1, String textv2) {
		try {
			Constructor constructor = diffMethod.getConstructor(new Class[]{String.class, String.class});
			return (Diff)constructor.newInstance(textv1, textv2);
		} catch (Exception e) {
			e.printStackTrace();
			return new PonsDiff(textv1, textv2);
		}
	}

	public static String[] getShortestDiffPath(String[] files, Class diffMethod) {

		//System.err.println("Finding shortest diff path...");

		ArrayList<String> shortestPath = new ArrayList<String>();

		int currentFile = files.length - 1;
		shortestPath.add(files[currentFile]);

		while (currentFile > 0) {
			int minEdits = Integer.MAX_VALUE;
			int closest_i = currentFile-1;
			String n = Document.fileReader(files[currentFile]);

			for (int i = currentFile-1; i>=0; i--) {
				String o = Document.fileReader(files[i]);
				if (Math.abs(o.length() - n.length()) <= minEdits) {
					Diff diff = createDiff(diffMethod, o, n);
					int editcount = getEditDistance(diff.getDiffs());
					if (editcount < minEdits || editcount == 0) {
						minEdits = editcount;
						closest_i = i;
					}
				}
			}
			currentFile = closest_i;
			shortestPath.add(0, files[currentFile]);
		}

		//System.err.println("\nDone.");

		return (String[])(shortestPath.toArray(new String[shortestPath.size()]));
	} 

	public static int getEditDistance(Operation[] ops) {
		int count = 0;
		for (int i=0; i<ops.length; i++)
			count += ops[i].length;
		return count;
	}

	public static String toString(Operation[] ops) {
		String s = "";
		for (Operation op : ops) {
			s += op.toString() + "\n";
		}
		return s;	
	}

	public static boolean equals(Operation[] ops1, Operation[] ops2) {
		if (ops1.length != ops2.length)
			return false;
		for (int i=0; i<ops1.length; i++) {
			if (ops1[i].type != ops1[i].type)
				return false;
			if (ops1[i].from != ops1[i].from)
				return false;
			if (ops1[i].length != ops1[i].length)
				return false;
		}
		return true;
	}

}
