/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diff.methods;

import java.lang.reflect.Constructor;

import fr.aviz.diff.Diff;
import fr.aviz.diff.DiffChecker;
import fr.aviz.diff.DiffUtils;

/**
 * 
 * This uses PonsDiff and delegates to MyersDiff when the result is not correct,
 * which currently happens when text moves.
 * 
 * @author dragice
 *
 */
public class BestDiff implements Diff {

	Operation[] ops;
	String output;

	public BestDiff(String o, String n) {

		output = "";

		if (!tryMethod(PonsDiff.class, o, n)) {
			if (!tryMethod(RatcliffDiff.class, o, n))
				tryMethod(MyersDiff.class, o, n);
			System.err.println(output);
		}
	}

	public boolean tryMethod(Class method, String o, String n) {
		output += "Trying " + method.getSimpleName();
		try {
			Constructor constructor = method.getConstructor(new Class[]{String.class, String.class});
			Diff diff = (Diff)constructor.newInstance(o, n);
			ops = diff.getDiffs();
			if (!DiffChecker.check(o, n, ops)) { 
				output += ": diff incorrect -> ";
				return false;
			}
		}
		catch (Exception e) {
			//e.printStackTrace();
			output += ": got an exception -> ";
			return false;
		}
		output += ": OK.";
		return true;
	}

	@Override
	public Operation[] getDiffs() {
		return ops;
	}

	public String toString() {
		return DiffUtils.toString(getDiffs());
	}

}
