/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.gui;

import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fr.aviz.animation.AnimationListener;
import fr.aviz.animation.AnimationTimer;
import fr.aviz.diff.Diff;
import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.Settings;
import fr.aviz.diffanimation.Document.Version;
import fr.aviz.utils.DocumentFactory;

public class JAnimationSettings extends JDialog implements ItemListener {
	//
	DiffAnimator diffAnimator;
	Checkbox v1_v2;
	Checkbox v1_v1v2;
	Checkbox v1v2_v2;
	Settings settings;
	boolean manualAnimationControl = false;

	
	/**
	 * @param directory
	 */
	public JAnimationSettings(DiffAnimator diffAnimator, JFrame owner) {
		super(owner);
		this.diffAnimator = diffAnimator;
		this.settings = diffAnimator.settings;
		addComponents();
		pack();
		setTitle("Settings");
		setVisible(true);
	}

	
	
	
	protected void addComponents() {
		Container c = getContentPane();
		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
		
		c.add(diffMethodComponent());
		c.add(animationEnabledComponent());
		//c.add(animatedVersionsComponent());
		c.add(rendererSettingsComponent());
	}
	
	protected Component animationEnabledComponent() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder(5, 10, 30, 10));
		panel.setLayout(new BorderLayout());
		final JCheckBox animationEnabled = new JCheckBox("Enable animations", settings.animationEnabled);
		animationEnabled.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				settings.animationEnabled = animationEnabled.isSelected();
			}
	    });
		panel.add(animationEnabled, BorderLayout.CENTER);
		return panel;
		}
	
	
	protected Component animatedVersionsComponent() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(1, 3));
		CheckboxGroup cbg = new CheckboxGroup();
		
		v1_v2 = new Checkbox("V1 -> V2", cbg, true);
		v1_v2.addItemListener(this);
		v1_v1v2 = new Checkbox("V1 -> V1V2", cbg, false);
		v1_v1v2.addItemListener(this);
		v1v2_v2 = new Checkbox("V1V2 -> V2", cbg, false);
		v1v2_v2.addItemListener(this);
		
		panel.add(v1_v2);
		panel.add(v1_v1v2);
		panel.add(v1v2_v2);
		
		return panel;
	}
	
	protected Component diffMethodComponent() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder(30, 10, 5, 10));
		panel.setLayout(new BorderLayout());
		final JComboBox diffMethod = new JComboBox(Diff.METHOD_NAMES);
		for (int i=0; i<Diff.METHODS.length; i++)
			if (settings.diffMethod == Diff.METHODS[i])
				diffMethod.setSelectedIndex(i);
		diffMethod.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (diffMethod.getSelectedIndex() == -1)
					return;
				Class oldmethod = settings.diffMethod;
				settings.diffMethod = Diff.METHODS[diffMethod.getSelectedIndex()];
				if (settings.diffMethod != oldmethod) {
					repaint();
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							// recompute everything
							setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
							diffAnimator.rebuildAllDocuments();
							setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						}
					});
				}
			}
		});
		panel.add(new JLabel("Diff method"), BorderLayout.WEST);
		panel.add(diffMethod, BorderLayout.CENTER);
		return panel;
	}
	
	public void itemStateChanged(ItemEvent event) {
		if (event.getItemSelectable() == v1_v2) { 
			diffAnimator.getDocument().setStartEndVersion(Version.V1, Version.V2);
			settings.vstart = Version.V1;
			settings.vend = Version.V2;
		} else if (event.getItemSelectable() == v1_v1v2) { 
	    	diffAnimator.getDocument().setStartEndVersion(Version.V1, Version.V1_V2);  
	    	settings.vstart = Version.V1;
	    	settings.vend = Version.V1_V2;
	    } else if (event.getItemSelectable() == v1v2_v2) { 
	    	diffAnimator.getDocument().setStartEndVersion(Version.V1_V2, Version.V2);
	    	settings.vstart=  Version.V1_V2;
	    	settings.vend = Version.V2;
	    }
	} 

	
	/**
	 * Renderer settings: main panel
	 */
	protected JPanel rendererSettingsComponent() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(3, 1));
		panel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Render settings"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
			
		panel.add(animationSettingsPanel());
		panel.add(textSettingsPanel());
		return panel;
	}
	
	
	
	
	
	/**
	 *  Animation settings (speed)
	 */
	protected JPanel animationSettingsPanel() {
		JPanel panimation = new JPanel();
		panimation.setLayout(new GridBagLayout());
		panimation.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Animation"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		//final JCheckBox slow = new JCheckBox("Slow animations");
		//slow.addActionListener(new ActionListener() {
		//	public void actionPerformed(ActionEvent e) {
		//		updateSpeed(slow.isSelected());
		//	}
		//});
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		
		JLabel speedLabel = new JLabel("Speed", JLabel.CENTER);
        speedLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        
		final JRealSlider speed = new JRealSlider(-1, 0.7); 
		final JCheckBox autoSpeed = new JCheckBox("Adaptive", settings.autoSpeed);

		speed.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				double speedRatio = Math.pow(10, speed.getDoubleValue()); 
				updateSpeed(speedRatio, autoSpeed.isSelected());
//System.err.println(diffAnimator.getSpeedNormalizer().getMotionMagnitude() + " " + speedRatio);
			}        	
	    });
		autoSpeed.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				double speedRatio = Math.pow(10, speed.getDoubleValue()); 
				updateSpeed(speedRatio, autoSpeed.isSelected());
			}        	
	    });
		speed.setDoubleValue(0);
		//updateSpeed(speed.getDoubleValue(), autoSpeed.isSelected());

		c.gridx = 0;
		c.gridy = 1;
		panimation.add(speedLabel,c);
		c.gridx = 1;
		panimation.add(speed, c);
		c.gridx = 2;
		panimation.add(autoSpeed, c);
		 
		final JRealSlider animstate = new JRealSlider(0, 1); 
		animstate.setDoubleValue(0);
		animstate.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				updateAnimationState(animstate.getDoubleValue());
			}        	
	    });
		animstate.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				manualAnimationControl = true;
			}
			public void mouseReleased(MouseEvent e) {
				manualAnimationControl = false;
			}
		});
		AnimationTimer.addListener(new AnimationListener() {
			public void animateEvent() {
				if (!manualAnimationControl) {
					if (diffAnimator.getDocument() != null)
						animstate.setDoubleValue(diffAnimator.getDocument().getAnimationState());
				}
			}
		});
		
		//c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		//panimation.add(animLabel);
		panimation.add(animstate,c);
		
		
		return panimation;
	}
	
	protected void updateSpeed(boolean slow, boolean autoAdapt) {
		settings.speed = slow ? 0.2 : 1;
		settings.autoSpeed = autoAdapt;
		if (diffAnimator.getDocument() != null)
			diffAnimator.getDocument().updateSpeed();
	}
	
	protected void updateSpeed(double value, boolean autoAdapt) {
		settings.speed = value;
		settings.autoSpeed = autoAdapt;
		if (diffAnimator.getDocument() != null)
			diffAnimator.getDocument().updateSpeed();
	}
	
	protected void updateAnimationState(double value) {
		if (manualAnimationControl) {
			diffAnimator.emptyAnimationQueue();
			diffAnimator.setAnimationState(value, true);
			diffAnimator.getOverview().animateEvent();
		}
	}
	
	
	/**
	 *  Text settings (color, transparency, highlights)
	 */
	protected JPanel textSettingsPanel() {
		JPanel ptextsettings = new JPanel();
		ptextsettings.setLayout(new BorderLayout());
		ptextsettings.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Text rendering"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		
		final JComboBox textFont = new JComboBox(Settings.TEXT_FONT_NAMES);
		textFont.setSelectedIndex(settings.textFontIndex);
		textFont.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				settings.textFontIndex = textFont.getSelectedIndex();
				settings.nominalTextLayout = null; // this will force recalculation
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				DocumentFactory.documentsFontChanged();
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				diffAnimator.repaint();
				diffAnimator.getOverview().repaint();
			}
		});
				
		JPanel pcheckboxes = new JPanel();
		pcheckboxes.setLayout(new GridLayout(3,2));
		
		final JCheckBox highlights = new JCheckBox("Text highlight", settings.highlights);
		pcheckboxes.add(highlights, BorderLayout.SOUTH);
		highlights.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateTextHighlights(highlights.isSelected());
				diffAnimator.repaint();
			}
		});
		final JCheckBox overviewHighlights = new JCheckBox("Sidebar highlight", settings.overviewHighlights);
		pcheckboxes.add(overviewHighlights, BorderLayout.SOUTH);
		overviewHighlights.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				settings.overviewHighlights = overviewHighlights.isSelected();
				diffAnimator.repaint();
			}
		});
		final JCheckBox colors = new JCheckBox("Text color", settings.colors);
		pcheckboxes.add(colors, BorderLayout.SOUTH);
		colors.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateTextColors(colors.isSelected());
				diffAnimator.repaint();
			}
		});
		final JCheckBox ctransient = new JCheckBox("Transient", settings.transientHighlights);
		pcheckboxes.add(ctransient, BorderLayout.CENTER);
		ctransient.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateTextTransient(ctransient.isSelected());
				diffAnimator.repaint();
			}
		});
		final JCheckBox transparency = new JCheckBox("Transparency", settings.transparency);
		pcheckboxes.add(transparency, BorderLayout.CENTER);
		transparency.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateTextTransparency(transparency.isSelected());
				diffAnimator.repaint();
			}
		});
		
		ptextsettings.add(textFont, BorderLayout.NORTH);
		ptextsettings.add(pcheckboxes, BorderLayout.CENTER);
		
		return ptextsettings;
	}

	protected void updateTextColors(boolean colors) {
		settings.colors = colors;
	}
	
	protected void updateTextTransparency(boolean transparency) {
		settings.transparency = transparency;
	}

	protected void updateTextHighlights(boolean highlights) {
		settings.highlights = highlights;
	}
	
	protected void updateTextTransient(boolean ctransient) {
		settings.transientHighlights = ctransient;
	}

	
	
	public static class JRealSlider extends JSlider {
		double min, max;
		/**
		 * Creates a slider with a specified min and max values
		 * @param min the min value
		 * @param max the max value
		 */
		public JRealSlider(double min, double max) {
			super(0, 1000);
			this.min = min;
			this.max = max;
			// not too big
	        setPreferredSize(new Dimension(100, getPreferredSize().height));
		}
		/**
		 * @return the current value
		 */
		public double getDoubleValue() {
			return min + (getValue() / 1000.0 * (max - min));
		}
		/**
		 * Sets the current value
		 * @param value the new value
		 */
		public void setDoubleValue(double value) {
			setValue((int)((value - min) / (max - min) * 1000));
		}
	}
}
