/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.gui;

import java.awt.Cursor;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.SwingUtilities;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.gui.JRevisionSelector.RevisionFile;


public class JLoadDirectory {
	DiffAnimator diffAnimator;

	protected String keyName = "";
	
	JDialog owner;
	JList revisionList;
	DefaultListModel revisionListElements;
	static final String filenamePattern = "^(.*)(\\D)(\\d+)(\\.txt)$";

	// The key here is RevisionFile.name
	Hashtable<String, ArrayList<RevisionFile>> revisionFiles = new Hashtable<String, ArrayList<RevisionFile>>();
	

	
	/**
	 * @param directory
	 */
	public JLoadDirectory(DiffAnimator diffAnimator, JDialog owner) {
		this.diffAnimator = diffAnimator;
		this.owner = owner;
	}
	

	
	
	public void loadFiles(File file) {
		revisionListElements = new DefaultListModel();
		revisionList = new JList(revisionListElements);
		parseFilenames(file.getParentFile());
		updateName(keyName);
	}
	
	
	/**
	 * Parse all files in a specific directory
	 * @param directory
	 */
	protected void parseFilenames(File directory) {
		revisionListElements.removeAllElements();
		revisionList.removeAll();
		revisionFiles.clear();
		
		// List files
		String[] files = directory.list();
		
		// Examine each filename
		String filename;
		String filepath;
		File file;
		Pattern pattern = Pattern.compile(filenamePattern);
	    Matcher matcher;
		for (int i=0; i<files.length; i++) {
			filename = files[i];
			filepath = directory.getPath() + File.separator + filename;
			file = new File(filepath);
			// Is the file a directory?
			if (file.isDirectory()) {
				// if yes, support recursion
				parseFilenames(file);
			} else {
				// if not, see if this filename matches the pattern
				matcher = pattern.matcher(filename);
				if (matcher.find()) {
//System.out.println("  found [" + matcher.group(1) + "] [" + matcher.group(2) + "] [" + matcher.group(3) + "]");
					// create a RevisionFile object
					RevisionFile rf = new RevisionFile();
					rf.fullFilename = filepath;
					rf.name = matcher.group(1) + matcher.group(2);
					keyName = rf.name;
					rf.revision = Integer.parseInt(matcher.group(3));
					   // System.out.println(rf.fullFilename);
					// add this object to the list
					ArrayList<RevisionFile> list;
					if (revisionFiles.containsKey(rf.name)) {
						list = revisionFiles.get(rf.name);
					} else {
						list = new ArrayList<RevisionFile>();
						revisionFiles.put(rf.name, list);
					}
					list.add(rf);
				}
			}
		}
		
		// Finally, sort the revision lists
		ArrayList<RevisionFile> list;
		for (String name : revisionFiles.keySet()) {
			list = revisionFiles.get(name);
			Collections.sort(list, new Comparator<RevisionFile>() {
				@Override
				public int compare(RevisionFile r1, RevisionFile r2) {
					return r1.revision - r2.revision;
				}
			});
		}
	}
	
	
	protected void updateName(String newName) {
		//currentRevision = null;
		revisionList.ensureIndexIsVisible(revisionList.getModel().getSize() - 1);
		
		// Clear diffAnimator
		diffAnimator.clear();
		diffAnimator.repaint();
		
		final String[] filenames = getAllFilenames(newName);
		if (filenames.length == 0)
			return;
		
		final String nname = newName;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				
				// Update diffAnim & pre-build documents
				owner.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				diffAnimator.newFiles(getAllFilenames(nname));
				diffAnimator.getTimeline().newFiles();
				diffAnimator.repaint();
				diffAnimator.getTimeline().repaint();
				diffAnimator.getOverview().repaint();
				owner.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				// Programmatically perform an animation between the second-last and the last version
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						revisionList.setSelectedIndex(revisionList.getModel().getSize() - 1);
						diffAnimator.requestFocus();
						diffAnimator.repaint();
						diffAnimator.startAnimation(filenames[0], filenames[0]);
						
					}
				});
			}
		});		
	}
	
	
	// Returns all the revision files
	private String[] getAllFilenames(String name) {
		if (name == null)
			return new String[] {};
		ArrayList<RevisionFile> revs = revisionFiles.get(name);
		String[] files = new String[revs.size()];
		for (int i = 0; i<files.length; i++)
			files[i] = revs.get(i).fullFilename;
		return files;
	}
	
	
}
