/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Insets;
import java.awt.Dialog.ModalityType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Random;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import fr.aviz.utils.QueryAPITask;


public class JProgressPanel extends JPanel implements ActionListener, PropertyChangeListener {

	private JProgressBar progressBar;
	private JButton startButton;
	private JTextArea taskOutput;
	private QueryAPITask task;
	private JDialog dialog;
	

	public JProgressPanel() {
		super(new BorderLayout());

		//Create the demo's UI.
		startButton = new JButton("Start");
		startButton.setActionCommand("start");
		startButton.addActionListener(this);

		progressBar = new JProgressBar(0, 100);
		progressBar.setValue(0);
		progressBar.setStringPainted(true);

		taskOutput = new JTextArea(5, 20);
		taskOutput.setMargin(new Insets(5,5,5,5));
		taskOutput.setEditable(false);

		JPanel panel = new JPanel();
		panel.add(startButton);
		panel.add(progressBar);

		add(panel, BorderLayout.PAGE_START);
		add(new JScrollPane(taskOutput), BorderLayout.CENTER);
		setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));

	}
	
	/**
	 * Invoked when the user presses the start button.
	 */
	public void actionPerformed(ActionEvent evt) {
		startButton.setEnabled(false);
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		//Instances of javax.swing.SwingWorker are not reusuable, so
		//we create new instances as needed.
		task = new QueryAPITask();
		task.addPropertyChangeListener(this);
		task.execute();
	}

	/**
	 * Invoked when task's progress property changes.
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if ("progress" == evt.getPropertyName()) {
			int progress = (Integer) evt.getNewValue();
			progressBar.setValue(progress);
			taskOutput.append(String.format(
					"Completed %d%% of task.\n", task.getProgress()));
			
			//if (task.isDone())
			//	dialog.dispose();
		} 
		
		if ("state" == evt.getPropertyName())
			if (task.isDone()) {
				dialog.setVisible(false);
				dialog.setModal(false);
			}
	}


	/**
	 * Create the GUI and show it. As with all GUI code, this must run
	 * on the event-dispatching thread.
	 */
	static void createAndShowGUI() {
		//Create and set up the window.
		JDialog dialog = new JDialog(null, "Query Wikipedia API", ModalityType.APPLICATION_MODAL);
		dialog.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

		//Create and set up the content pane.
		JComponent newContentPane = new JProgressPanel();
		newContentPane.setOpaque(true); //content panes must be opaque
		dialog.setContentPane(newContentPane);

		//Display the window.
		dialog.pack();
		dialog.setVisible(true);
	}

	public static void main(String[] args) {
		//Schedule a job for the event-dispatching thread:
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}

	
}
