/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import fr.aviz.diffanimation.DiffAnimator;

public class JRevisionSelector extends JDialog implements ActionListener {

	static final String DATA_DIR = "data";
	static final boolean FLAT_LIST = false;
	static final boolean SHOW_REVISIONS = false;
	
	public static final String DOWNLOAD_DIR = "data/wikidownloads";

	static final String filenamePattern = "^(.*)(\\D)(\\d+)(\\.txt)$";

	static class RevisionFile {
		String fullFilename; // e.g., formattedText2.txt
		String name;         // e.g., formattedText
		int revision;        // e.g., 2
		public String toString() {
			return revision + "";
		}
	}
	
	static JLoadDirectory loader; 	
	protected JButton loadbutton;
	protected JButton wikipediabutton;
	protected boolean done = false;
	protected JWikipediaPanel wikipanel;
	
	
	
	// The key here is RevisionFile.name
	Hashtable<String, ArrayList<RevisionFile>> revisionFiles = new Hashtable<String, ArrayList<RevisionFile>>();
	
	//
	DiffAnimator diffAnimator;
	String currentName = null; // text name
	RevisionFile currentRevision = null; // revision
	
	// Components
	JList nameList;
	DefaultListModel nameListElements;
	JTree nameTree;
	DefaultMutableTreeNode nameTreeElements;
	JList revisionList;
	DefaultListModel revisionListElements;
	
	/**
	 * 
	 * @param directory
	 */
	public JRevisionSelector(DiffAnimator diffAnimator, String directory, final String defaultFile, JFrame owner) {
		super(owner);
		this.diffAnimator = diffAnimator;
		loader = new JLoadDirectory(diffAnimator, this);
		File dir = new File(directory);
		if (dir.exists() && dir.isDirectory()) {
			parseFilenames(dir);
			addComponents();
			pack();
			setTitle("Examples");
			//setVisible(true);
			/*SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					try {
						nameList.setSelectedIndex(getIndex(defaultFile));
						nameTree.setSelectionPath(new TreePath(getLeaf(nameTreeElements, defaultFile).getPath()));
					} catch (Exception e) {
						System.err.println("Cannot find the default file " + defaultFile);
					}
					revisionList.requestFocus();
				}
			});*/
			setSize(300, 500);
		} else {
			System.err.println("Directory " + directory + " not found.");
			System.exit(0); // ben ouais, ca rigole pas.  
			                // hehehe... t'es comme ca toi...
		}
	}
	
	
	public void addDirectory(String directory) {
		File dir = new File(directory);
		if (dir.exists() && dir.isDirectory()) {
			parseFilenames(dir);
			pack();
		}
	}
	
	/**
	 * @param directory
	 * @param revisionFiles
	 */
	protected void parseFilenames(File directory) {
		// List files
		String[] files = directory.list();
		
		// Examine each filename
		String filename;
		String filepath;
		File file;
		Pattern pattern = Pattern.compile(filenamePattern);
	    Matcher matcher;
		for (int i=0; i<files.length; i++) {
			filename = files[i];
			filepath = directory.getPath() + File.separator + filename;
			   //System.out.println(filepath);
			file = new File(filepath);
			// Is the file a directory?
			if (file.isDirectory()) {
				// if yes, support recursion
				parseFilenames(file);
			} else {
				// if not, see if this filename matches the pattern
				matcher = pattern.matcher(filename);
				if (matcher.find()) {
//System.out.println("  found [" + matcher.group(1) + "] [" + matcher.group(2) + "] [" + matcher.group(3) + "]");
					// create a RevisionFile object
					RevisionFile rf = new RevisionFile();
					rf.fullFilename = filepath;
					rf.name = matcher.group(1) + matcher.group(2);
					rf.revision = Integer.parseInt(matcher.group(3));
					   // System.out.println(rf.fullFilename);
					// add this object to the list
					ArrayList<RevisionFile> list;
					if (revisionFiles.containsKey(rf.name)) {
						list = revisionFiles.get(rf.name);
					} else {
						list = new ArrayList<RevisionFile>();
						revisionFiles.put(rf.name, list);
					}
					list.add(rf);
				}
			}
		}
		
		// Finally, sort the revision lists
		ArrayList<RevisionFile> list;
		for (String name : revisionFiles.keySet()) {
			list = revisionFiles.get(name);
			Collections.sort(list, new Comparator<RevisionFile>() {
				@Override
				public int compare(RevisionFile r1, RevisionFile r2) {
					return r1.revision - r2.revision;
				}
			});
		}
	}
	
	/**
	 * 
	 */
	protected void addComponents() {
		Container c = getContentPane();
		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
		
		// -- REVISION LIST
		
		revisionListElements = new DefaultListModel();
		revisionList = new JList(revisionListElements);

		// -- DOCUMENTS (FLAT LIST)
		
		nameListElements = new DefaultListModel();
		Set<String> names = revisionFiles.keySet();
		ArrayList<String> anames = new ArrayList<String>(names);
		Collections.sort(anames, new Comparator<String>() {
			@Override
			public int compare(String n1, String n2) {
				return n1.compareToIgnoreCase(n2);
			}
		});
		for (String name : anames)
			nameListElements.addElement(name);
		nameList = new JList(nameListElements);
		nameList.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		nameList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		nameList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (nameList.getSelectedIndex() == -1)
							updateName(null);
						else {
							String newName = (String)nameListElements.get(nameList.getSelectedIndex());
							updateName(newName.trim());
						}
					}
				});
			}
		});
		nameList.setRequestFocusEnabled(false);

		// -- DOCUMENTS (TREE)
		
		nameTreeElements = new DefaultMutableTreeNode(DATA_DIR);
		Collections.sort(anames, new Comparator<String>() {
			@Override
			public int compare(String n1, String n2) {
				String fullName1 = revisionFiles.get(n1).get(0).fullFilename;
				String fullName2 = revisionFiles.get(n2).get(0).fullFilename;
				return fullName1.compareToIgnoreCase(fullName2);
			}
		});

		for (String name : anames) {
			RevisionFile rev0 = revisionFiles.get(name).get(0);
			String fullname = rev0.fullFilename;
			StringTokenizer st = new StringTokenizer(fullname, File.separator);
			DefaultMutableTreeNode currentFolder = nameTreeElements;
			int count = st.countTokens();
			for (int i=0; st.hasMoreTokens(); i++) {
				String foldername = st.nextToken();
				if (i >= 2 && i < count-1) {
					DefaultMutableTreeNode newFolder = getChild(currentFolder, foldername);
					if (newFolder == null) {
						newFolder = new DefaultMutableTreeNode(foldername);
						currentFolder.add(newFolder);
					}
					currentFolder = newFolder;
				}
			}
			currentFolder.add(new DefaultMutableTreeNode(name));
		}
		nameTree = new JTree(nameTreeElements);
		nameTree.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		nameTree.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		nameTree.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						DefaultMutableTreeNode selection = (DefaultMutableTreeNode)nameTree.getSelectionPath().getLastPathComponent();
						if (selection == null || !selection.isLeaf())
							updateName(null);
						else {
							String newName = (String)selection.getUserObject();
							updateName(newName.trim());
						}
					}
				});
			}
		});
		nameTree.setRequestFocusEnabled(false);
		//
		
		JPanel p1 = new JPanel();
		p1.setLayout(new BorderLayout(5, 5));
		p1.add(new JLabel("Texts"), BorderLayout.NORTH);
		JScrollPane sp_list = new JScrollPane(nameList);
		JScrollPane sp_tree = new JScrollPane(nameTree);
		
		if (FLAT_LIST)
			p1.add(sp_list, BorderLayout.CENTER);
		else
			p1.add(sp_tree, BorderLayout.CENTER);
		
		
		// -- REVISION LIST
		
		JPanel p2 = new JPanel();
		p2.setLayout(new BorderLayout(5, 5));
		p2.add(new JLabel("Revisions"), BorderLayout.NORTH);
//		revisionList.setPreferredSize(new Dimension(130, 340));
		revisionList.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		revisionList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		revisionList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						if (revisionList.getSelectedIndex() == -1) {
							updateRevision(null);
						} else {
							RevisionFile newRevision = (RevisionFile)revisionListElements.get(revisionList.getSelectedIndex());
							updateRevision(newRevision);
						}
					}
				});
			}
		});
			
		if (SHOW_REVISIONS) {
			JScrollPane sp2 = new JScrollPane(revisionList);
			p2.add(sp2, BorderLayout.CENTER);
			
			JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
			sp.setTopComponent(p1);
			sp.setBottomComponent(p2);
			c.add(sp, BorderLayout.CENTER);
			sp.setDividerLocation(0.75);
		} else {
			c.add(p1, BorderLayout.CENTER);
		}
		
		
		// Load document in a specified directory
		loadbutton = new JButton("Load directory");
		loadbutton.addActionListener(this);
		c.add(loadbutton);
		
		// Load revisions from Wikipedia
		wikipediabutton = new JButton("Load wikipedia article");
		wikipediabutton.addActionListener(this);
		c.add(wikipediabutton);
	}
	
	protected void updateName(String newName) {
		if (newName != null && newName.equals(currentName))
			return;
		
		// Update revision list
		revisionListElements.removeAllElements();
		if (newName != null) {
			for (RevisionFile rf : revisionFiles.get(newName))
				revisionListElements.addElement(rf);
		}
		currentName = newName;
		currentRevision = null;
		revisionList.ensureIndexIsVisible(revisionList.getModel().getSize() - 1);
		
		// Clear diffAnimator
		diffAnimator.clear();
		diffAnimator.repaint();
		
		final String[] filenames = getAllFilenames(currentName);
		if (filenames.length == 0)
			return;
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				
				// Update diffAnim & pre-build documents
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				diffAnimator.newFiles(getAllFilenames(currentName));
				diffAnimator.getTimeline().newFiles();
				diffAnimator.repaint();
				diffAnimator.getTimeline().repaint();
				diffAnimator.getOverview().repaint();
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				// Programmatically perform an animation between the second-last and the last version
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						revisionList.setSelectedIndex(revisionList.getModel().getSize() - 1);
						diffAnimator.requestFocus();
						diffAnimator.repaint();
					}
				});
			}
		});		
	}
	
	// Returns all the revision files
	private String[] getAllFilenames(String name) {
		if (name == null)
			return new String[] {};
		ArrayList<RevisionFile> revs = revisionFiles.get(name);
		String[] files = new String[revs.size()];
		for (int i = 0; i<files.length; i++)
			files[i] = revs.get(i).fullFilename;
		return files;
	}
	
	protected void updateRevision(RevisionFile newRevision) {
		if (newRevision == currentRevision)
			return;
		if (currentRevision == null) {
			// -- First revision - skip
			diffAnimator.startAnimation(newRevision.fullFilename, newRevision.fullFilename);
		} else if (newRevision != null) {
			// --- New revision
			RevisionFile before = currentRevision;
			RevisionFile after = newRevision;
/*			boolean revert = after.revision < before.revision;
			if (revert) {
				RevisionFile tmp = before;
				before = after;
				after = tmp;
			}
*/			// Perform the document animation
			diffAnimator.queueAnimation(before.fullFilename, after.fullFilename);
		}
		currentRevision = newRevision;
	}
	
	public int getIndex(String name) {
		for (int i=0; i<nameListElements.size(); i++) {
			if (((String)nameListElements.get(i)).equals(name))
				return i;
		}
		return 0;
	}

	static DefaultMutableTreeNode getChild(DefaultMutableTreeNode parent, String name) {
		for (Enumeration e = parent.children(); e.hasMoreElements();) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			if (name.equals(n.getUserObject()))
				return n;
		}
		return null;
	}
	
	static DefaultMutableTreeNode getLeaf(DefaultMutableTreeNode parent, String name) {
		DefaultMutableTreeNode child = getChild(parent, name);
		if (child != null && child.isLeaf())
			return child;
		for (Enumeration e = parent.children(); e.hasMoreElements();) {
			DefaultMutableTreeNode n = (DefaultMutableTreeNode)e.nextElement();
			child = getLeaf(n, name);
			if (child != null)
				return child;
		}
		return null;
	}
	
	
	@Override
	public void actionPerformed(ActionEvent e) {
		 if (e.getSource() == loadbutton) {
			 JFileChooser fileChooser = new JFileChooser(".");
			 int ret = fileChooser.showOpenDialog(null);
			 if (ret == JFileChooser.APPROVE_OPTION) {
				 File file = fileChooser.getSelectedFile();
				 System.out.println("Choosed file " + file.getPath());
           
				 // Load the full directory
				 loader.loadFiles(file);
			 } 
		 }
		 
		 else if (e.getSource() == wikipediabutton) {
			 wikipanel = new JWikipediaPanel();
			 done = false;
			 
			 final WikipediaDialog wiki = new WikipediaDialog();
			 wiki.addWindowListener( new WindowListener() {
				 public void windowClosed( WindowEvent e ) {
					 //launchAction(wiki.isCanceled());
					 String article = wikipanel.getArticle();
					 launchProgress(wiki.isCanceled(), article, wikipanel.getUrl(), loader);
				 }
				 public void windowActivated(WindowEvent e) {}
				 public void windowDeactivated(WindowEvent e) {}
				 public void windowDeiconified(WindowEvent e) {}
				 public void windowIconified(WindowEvent e) {}
				 public void windowOpened(WindowEvent e) {}
				 public void windowClosing(WindowEvent e) {}
			 });
			 wiki.showModalDialog();
			 
			
			 // Doesn't work on Mac:
			 /*
			 int ret = JOptionPane.showConfirmDialog(this, wikipanel, "Wikipedia article settings", JOptionPane.OK_CANCEL_OPTION);
			 
			 if ( ret == JOptionPane.OK_OPTION )
			 {
				 System.out.println(wikipanel.getUrl());
				 String article = wikipanel.getArticle();
				
				 // Create directory
				 String dir = DOWNLOAD_DIR + "/" + article;
				 File fdir = new File(dir);
				 if (!fdir.exists())
					 fdir.mkdirs();
				 
				 setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				 WikipediaArticleDownloader.downloadHistoryFromUrl(dir, article, wikipanel.getUrl(), true);
				 
				 File file = new File(dir+"/" + article);
				 loader.loadFiles(file);
				 setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			 }*/
			 
			 
			
			 
		 }
		
	}
	
	private void launchProgress(boolean canceled, String article, String url, JLoadDirectory loader) {
		if (!done) {
			if (!canceled) {
				final String articlef = article;
				final String wikiversion = wikipanel.getWikiApi();
				final String urlf = url;
				final JLoadDirectory loaderf = loader;
				
				final JProgressMonitorPanel progress = new JProgressMonitorPanel();
				javax.swing.SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						String dir = DOWNLOAD_DIR + "/" + articlef;
						
						// Clean up directory
						 File download = new File(dir);
						 DiffAnimator.deleteDirectory(download);
						 
						// Create output directory
							(new File(dir)).mkdirs();
						
						// Query API
						progress.startMonitor(dir, wikiversion, articlef, urlf, loaderf);
					}
				});
			}
			done = true;
		}
	}
	
	
	class WikipediaDialog extends JDialog {
		private boolean canceled;
		private JButton okButton;
		private JButton cancelButton;

		private ActionListener okCancelListener = new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				if (e.getSource() == cancelButton) {
					System.out.println("Cancel button");
					setCancel(true);
					dispose();
				}
				else if (e.getSource() == okButton) {
					System.out.println("ok button");
					setCancel(false);
					
					if (validateEntries())
						dispose();
				}
			}
		};

		private boolean validateEntries() {
			return wikipanel.validateEntries();
		}
		
		public WikipediaDialog() {
			initComponents();
			setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
			setModal(true);

			/*addWindowListener(new WindowListener() {
			public void windowClosing(WindowEvent e) {
				setCancel(true);
			}
			public void windowActivated(WindowEvent e) {}
			public void windowClosed(WindowEvent e) {}
			public void windowDeactivated(WindowEvent e) {}
			public void windowDeiconified(WindowEvent e) {}
			public void windowIconified(WindowEvent e) {}
			public void windowOpened(WindowEvent e) {}
			});*/
		}

		private void setCancel(boolean canceled) {
			System.out.println("Set cancel");
			this.canceled = canceled;
		}

		private void initComponents() {
			Container c = this.getContentPane();
			c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
			
			c.add(wikipanel);
			c.add(buttonPanel());
		}

		private JPanel buttonPanel() {
			JPanel panel = new JPanel();
			
			okButton = new JButton("Ok");
			cancelButton = new JButton("Cancel");
			panel.add(okButton);
			panel.add(cancelButton);
			okButton.addActionListener(okCancelListener);
			cancelButton.addActionListener(okCancelListener);
			
			return panel;
		}
		
		public boolean isCanceled() {
			return canceled;
		}

		public boolean showModalDialog() {
			setModal(false);
			pack();
			setVisible(true);
			return !canceled;
		}
	}
	
	
	// For testing
	public static void main(String[] args) {
		new JRevisionSelector(null, "./" + DATA_DIR, "", null);
	}
}
