/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.gui;

import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import fr.aviz.utils.SpringUtilities;

public class JWikipediaPanel extends JPanel implements ItemListener {
protected Color gray = new Color(220,220,220);
	
	protected Checkbox cb_title;
	protected JTextField ta_title = new JTextField(20);
	protected Checkbox cb_id;
	protected JTextField ta_id = new JTextField(7);
	
	
	protected Checkbox from_default;
	protected Checkbox from_id;
	protected Checkbox from_ts;
	protected JTextField tf_from_id  = new JTextField(7);
	protected JTextField tf_from_ts  = new JTextField(10);
	
	protected Checkbox to_default;
	protected Checkbox to_id;
	protected Checkbox to_ts;
	protected JTextField tf_to_id  = new JTextField(7);
	protected JTextField tf_to_ts  = new JTextField(10);
	protected JTextField max = new JTextField(4);
	
	protected Checkbox frwiki;
	protected Checkbox enwiki;
	
	JWikipediaPanel(){
		super();
		addComponents();
	}
	
	
	private void addComponents() {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(wikipediaPanel());
		this.add(articlePanel());
		this.add(panelRevisions());
	}
	
	
	private JPanel wikipediaPanel() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Wikipedia version"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
	
		
		CheckboxGroup cbg = new CheckboxGroup();
		enwiki = new Checkbox(" english", cbg, true);
		setActive(enwiki, true);
		enwiki.addItemListener(this);
		frwiki = new Checkbox(" french", cbg, false);
		setActive(frwiki, true);
		frwiki.addItemListener(this);
		panel.add(enwiki);
		panel.add(frwiki);
		
		return panel;
	}
	
	
	private JPanel articlePanel() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Article"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		
		panel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.BOTH;
		
		CheckboxGroup cbg = new CheckboxGroup();
		
		cb_title = new Checkbox(" Title", cbg, true);
		setActive(cb_title, ta_title, true);
		ta_title.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
		cb_title.addItemListener(this);
		cb_id = new Checkbox(" ID", cbg, false);
		cb_id.addItemListener(this);
		setActive(cb_id, ta_id, false);
		ta_id.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
		
		c.gridx = 0;
		c.gridy = 0;
		panel.add(cb_title,c);
		c.gridy = 1;
		panel.add(ta_title,c);
		c.insets = new Insets(0,10,0,10); 
		c.gridx = 1;
		c.gridy = 0;
		panel.add(cb_id,c);
		c.gridy = 1;
		panel.add(ta_id,c);
		
		return panel;
	}
	

	private JPanel panelRevisions() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Revisions"),
                BorderFactory.createEmptyBorder(5,5,5,5)));
		
		panel.add(panelfrom());
		panel.add(panelto());
		panel.add(panelMax());
		
		return panel;
		
	}

	
	public JPanel panelMax() {
		JPanel p = new JPanel();
		p.setBorder(BorderFactory.createCompoundBorder(
            	BorderFactory.createTitledBorder("Max (2-50)"),
            	BorderFactory.createEmptyBorder(5,5,5,5)));
		max.setText("10");
		setActive(max, true);
		p.add(max);
		return p;
	}

	public JPanel panelfrom() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createCompoundBorder(
                	BorderFactory.createTitledBorder("From"),
                	BorderFactory.createEmptyBorder(5,5,5,5)));
		panel.setLayout(new SpringLayout());
		
		CheckboxGroup cbg = new CheckboxGroup();
		// default
		from_default = new Checkbox("Default (max revisions)", cbg, true);
		setActive(from_default, true);
		from_default.addItemListener(this);
		
		// timestamp
		from_ts = new Checkbox("Date (YYYY-MM-DD)", cbg, false);
		from_ts.addItemListener(this);
		setActive(from_ts, tf_from_ts, false);
		tf_from_ts.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
		
		// id
		from_id = new Checkbox("ID", cbg, false);
		from_id.addItemListener(this);
		setActive(from_id, tf_from_id, false);
		tf_from_id.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
	
		
		// layout
		panel.add(from_default);
		panel.add(new JLabel(""));
		panel.add(from_ts);
		panel.add(tf_from_ts);
		panel.add(from_id);
		panel.add(tf_from_id);
		
		SpringUtilities.makeCompactGrid(panel,
                3, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad

		
		return panel;
	}
	
	
	public String getArticle() {
		String article = "http://";
		if (cb_title.getState()) return ta_title.getText();
		else return ta_id.getText();
	}
	
	public String getWikiApi() {
		if (frwiki.getState()) return "http://fr.wikipedia.org";
		else return "http://en.wikipedia.org";
	}
		
	
	public String getUrl() {
		String version;
		if (frwiki.getState()) version = "fr";
		else version = "en";
		
		String article = "";
		if (cb_title.getState()) {
			article = "titles=" + ta_title.getText();
		}
		else article = "pageids" + ta_id.getText();
		
		// The order start/end is inverted for the query
		String from = "";
		if (from_id.getState())
			from += "&rvendid=" + tf_from_id.getText();
		else if (from_ts.getState())
			from += "&rvend=" + tf_from_ts.getText().replaceAll("-", "") + "000000";
		
		String to = "";
		if (to_id.getState())
			to += "&rvstartid=" + tf_to_id.getText();
		else if (to_ts.getState())
			to += "&rvstart=" + tf_to_ts.getText().replaceAll("-", "") + "000000";
		
		String limit = "";
		if (max.isEditable()) 
			limit = "&rvlimit=" + max.getText();
		else
			limit = "&rvlimit=50";
		
		String url = "http://" + 
			version + 
			".wikipedia.org/w/api.php?action=query&prop=revisions&" + 
			article +
			"&rvprop=ids&format=xml" + 
			from + to + limit;
			
		return url;
	}
	
	public JPanel panelto() {
		JPanel panel = new JPanel();
		panel.setBorder(BorderFactory.createCompoundBorder(
                	BorderFactory.createTitledBorder("To"),
                	BorderFactory.createEmptyBorder(5,5,5,5)));
		panel.setLayout(new SpringLayout());
		
		CheckboxGroup cbg = new CheckboxGroup();
		// default
		to_default = new Checkbox("Default (max revisions)", cbg, true);
		setActive(to_default, true);
		to_default.addItemListener(this);
		
		// timestamp
		to_ts = new Checkbox("Date (YYYY-MM-DD)", cbg, false);
		tf_to_ts.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
		setActive(to_ts,tf_to_ts, false);
		to_ts.addItemListener(this);
		
		// id
		to_id = new Checkbox("ID", cbg, false);
		tf_to_id.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
		setActive(to_id, tf_to_id, false);
		to_id.addItemListener(this);
		
		// layout
		panel.add(to_default);
		panel.add(new JLabel(""));
		panel.add(to_ts);
		panel.add(tf_to_ts);
		panel.add(to_id);
		panel.add(tf_to_id);
		
		SpringUtilities.makeCompactGrid(panel,
                3, 2, //rows, cols
                6, 6,        //initX, initY
                6, 6);       //xPad, yPad

		
		return panel;
	}
	
	private void setActive(Checkbox box, boolean active) {
		/*if (active) {
			box.setForeground(Color.black);
		}
		else {
			box.setForeground(Color.gray);
		}*/
	}
	
	private void setActive(JTextField text, boolean active) {
		if (active) {
			text.setBackground(Color.white);
			text.setForeground(Color.black);
			text.setEditable(true);
		}
		else {
			text.setBackground(gray);
			text.setForeground(Color.gray);
			text.setEditable(false);
		}
	}

	private void setActive(Checkbox box, JTextField text, boolean active) {
		setActive(box, active);
		setActive(text, active);
	}
	
	public boolean validateEntries() {
		if (cb_title.getState()) {
			if (ta_title.getText().isEmpty()) {
				JOptionPane.showMessageDialog(null, "Empty title", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
				return false;
			}
		}
		
		// Check id
		if (cb_id.getState()) {
			try {
				Integer.parseInt(ta_id.getText());
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(null, "Invalid article ID", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
				return false;
			}
		}
		
		
		// Check from-to IDs
		if (from_id.getState() || to_id.getState()) {
			int from = 0;
			int to = 0;
			
			try {
				if (from_id.getState())
					from = Integer.parseInt(tf_from_id.getText());
				if (to_id.getState())
					to = Integer.parseInt(tf_to_id.getText());
				if (from_id.getState() && to_id.getState())
					if (from >= to) {
						JOptionPane.showMessageDialog(null, "From ID has to be lower than To ID", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
						return false;
					}
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(null, "Invalid Revisions IDs", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
				return false;
			}
		}
		
		
		// Check from-to Date
		DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		if (from_ts.getState() || to_ts.getState()) {
			try {
				Date from = format.parse(tf_from_ts.getText());
				Date to = format.parse(tf_to_ts.getText());
				if (from_ts.getState() && to_ts.getState())
					if (from.after(to)) {
						JOptionPane.showMessageDialog(null, "From timestamp has to be before To timestamp", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
						return false;
					}
			}
			catch (ParseException e) {
				JOptionPane.showMessageDialog(null, "Invalid Revisions Timestamps", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
				return false;
			}
		}
		
		
		// Check max revisions
		if (max.isEditable()) {
			try {
				Integer.parseInt(max.getText());
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(null, "Invalid max revision", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
				return false;
			}
		}
		
		int m = Integer.parseInt(max.getText());
		if (m < 2 || m > 50) {
			JOptionPane.showMessageDialog(null, "Invalid max revision", "Wikiepdia API Query", JOptionPane.WARNING_MESSAGE);
			return false;
		}
		
		return true;
	}
	
	@Override
	public void itemStateChanged(ItemEvent e) {
		
		
		if (e.getSource() == cb_title) {
			setActive(cb_title, ta_title, true);
			setActive(cb_id, ta_id, false);
		}
		
		else if (e.getSource() == cb_id) {
			setActive(cb_title,ta_title, false);
			setActive(cb_id, ta_id, true);
		}
		
		
		else if (e.getSource() == from_default) {
			setActive(from_default, true);
			setActive(max, true);
			setActive(from_ts, tf_from_ts, false);
			setActive(from_id, tf_from_id, false);
		}
		
		else if (e.getSource() == from_ts) {
			setActive(from_default, false);
			if (!to_default.getState())
				setActive(max, false);
			setActive(from_ts, tf_from_ts, true);
			setActive(from_id, tf_from_id, false);	
		}
		
		else if (e.getSource() == from_id) {
			setActive(from_default, false);
			if (!to_default.getState())
				setActive(max, false);
			setActive(from_ts, tf_from_ts, false);
			setActive(from_id, tf_from_id, true);
		}
		
		
		else if (e.getSource() == to_default) {
			setActive(to_default, true);
			setActive(max, true);
			setActive(to_ts, tf_to_ts, false);
			setActive(to_id, tf_to_id, false);
		}
		
		else if (e.getSource() == to_ts) {
			setActive(to_default, false);
			if (!from_default.getState())
				setActive(max, false);
			setActive(to_ts, tf_to_ts, true);
			setActive(to_id, tf_to_id, false);
		}
		
		else if (e.getSource() == to_id) {
			setActive(to_default, false);
			if (!from_default.getState())
				setActive(max, false);
			setActive(to_ts, tf_to_ts, false);
			setActive(to_id, tf_to_id, true);
		}
	}
}
