/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.overview;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.overview.stabilization.GlobalMotionStabilizer;
import fr.aviz.diffanimation.overview.stabilization.ScrollingMemorizer;
import fr.aviz.diffanimation.overview.stabilization.StabilizationMethod;
import fr.aviz.diffanimation.overview.stabilization.TextAnchorStabilizer;
import fr.aviz.diffanimation.overview.stabilization.TopBottomStabilizer;

/**
 * 
 * @author dragice
 *
 */
public class ScrollingStabilizer {

	public enum methods {
		NONE,
		TEXT_ANCHOR,
		GLOBAL_MOTION
	};
	methods method = methods.GLOBAL_MOTION; 

	TopBottomStabilizer topBottomMethod;
	StabilizationMethod textAnchorMethod;
	StabilizationMethod globalMotionMethod;
	
	boolean enabled = true;
	boolean top_bottom_anchor_enabled = true; // depends on DiffOverView.allowThumbToBeOutside
	boolean use_top_bottom_anchor = false;
	float[] scrollingVector = new float[]{0, 0};
	
	public ScrollingStabilizer(DiffOverview overview, DiffAnimator detailview) {
		topBottomMethod = new TopBottomStabilizer(overview, detailview);
		textAnchorMethod = new ScrollingMemorizer(new TextAnchorStabilizer(overview, detailview));
		globalMotionMethod = new ScrollingMemorizer(new GlobalMotionStabilizer(overview, detailview));
		setMethod(method);
	}
	
	public void setMethod(methods method) {
		textAnchorMethod.setFeedbackEnabled(false);
		globalMotionMethod.setFeedbackEnabled(false);
		this.method = method;
		if (method == methods.TEXT_ANCHOR)
			textAnchorMethod.setFeedbackEnabled(true);
		else if (method == methods.GLOBAL_MOTION)
			globalMotionMethod.setFeedbackEnabled(true);
	}
	
	public void setTopBottomAnchorEnabled(boolean enabled) {
		this.top_bottom_anchor_enabled = enabled;
	}
	
	private StabilizationMethod getStabilizer() {
		if (use_top_bottom_anchor)
			return topBottomMethod;
		if (method == methods.TEXT_ANCHOR)
			return textAnchorMethod;
		if (method == methods.GLOBAL_MOTION)
			return globalMotionMethod;
		return null;
	}
	
	/**
	 * Called when new revisions have just been loaded and processed. The layout
	 * objects (paragraphs, blocks and words) have changed.
	 */
	public void documentChanged(DocumentAnimator doc) {
		if (top_bottom_anchor_enabled)
			topBottomMethod.documentChanged(doc);
		if (method == methods.TEXT_ANCHOR)
			textAnchorMethod.documentChanged(doc);
		else if (method == methods.GLOBAL_MOTION)
			globalMotionMethod.documentChanged(doc);
	}
	
	/**
	 * Called when the user has just finished scrolling manually (mouse up) 
	 */
	public void setViewOfInterest(boolean validate) {
		use_top_bottom_anchor = top_bottom_anchor_enabled && (topBottomMethod.thumbOnBottom() || topBottomMethod.thumbOnTop());
		textAnchorMethod.setFeedbackEnabled(method == methods.TEXT_ANCHOR && !use_top_bottom_anchor);
		getStabilizer().setViewOfInterest(validate);
	}
	
	/**
	 * Enables / disables stabilization.
	 * Called while the user is scrolling manually. 
	 */
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
	
	/**
	 * Called at every animation step
	 */
	public void stabilize() {
		if (!enabled)
			return;
		getStabilizer().stabilize();
	}
	
	public float[] getStartEndScrollingVector() {
		return getStabilizer().getStartEndScrollingVector();
	}
}
