/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.overview.stabilization;

import java.awt.geom.Point2D;
import java.util.ArrayList;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.Document;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.Document.Version;
import fr.aviz.diffanimation.TextBlock.TextWord;
import fr.aviz.diffanimation.overview.DiffOverview;

public class GlobalMotionStabilizer extends StabilizationMethod {
	
	public GlobalMotionStabilizer(DiffOverview overview, DiffAnimator detailview) {
		super(overview, detailview);
	}

	@Override
	public void documentChanged(DocumentAnimator doc) {
		super.documentChanged(doc);
		setViewOfInterest(true);
	}
	
	@Override
	public void setViewOfInterest(boolean validate) {

		if (document == null) {
			scrollingVector[0] = 0;
			scrollingVector[1] = 0;
			return;
		}
		
		if (!validate)
			return;
		
		// Check the direction of the animation
		Version startVersion, endVersion;
		float startT, endT;
		float startY, endY;
		if (document.getMotionAnimationState() < 0.5) {
			startVersion = document.getStartVersion();
			startT = 0;
			endVersion = document.getEndVersion();
			endT = 1;
		} else {
			startVersion = document.getEndVersion();
			startT = 1;
			endVersion = document.getStartVersion();
			endT = 0;
		}

		// Get all currently visible words
		float docy0 = overview.bar2doc(overview.thumbY());
		float docy1 = overview.bar2doc(overview.thumbY() + overview.thumbHeight());
		ArrayList<TextWord> visibleWords = document.getAllWords(docy0, docy1, true);
		
		// Compute the vertical displacement of these words
		float dy, weight;
		float dy_sum = 0, weight_sum = 0;
		for (TextWord tw : visibleWords) {
			float[] geom0 = tw.getPrecomputedGeometry(startVersion);
			float[] geom1 = tw.getPrecomputedGeometry(endVersion);
			dy = (geom1[1]+geom1[3])/2 - (geom0[1]+geom0[3])/2;
			// weight is proportional to the word's initial width
			weight = Math.abs(geom0[2]-geom0[0]);
			dy_sum += dy * weight;
			weight_sum += weight;
		}
		
		// Compute the equation of vertical movement
		float averageDy = 0;
		if (weight_sum > 0)
			averageDy = dy_sum / weight_sum;
		startY = docy0;
		endY = docy0 + averageDy;
		
		// Now update the scrolling vector 
		if (startT == 0) {
			scrollingVector[0] = startY;
			scrollingVector[1] = endY;
		} else if (startT == 1) {
			scrollingVector[0] = endY;
			scrollingVector[1] = startY;
		}
		
		// Take into account thumb range constraints
		scrollingVector[0] = getLocationWithinBounds(scrollingVector[0], 0);
		scrollingVector[1] = getLocationWithinBounds(scrollingVector[1], 1);
	
	}
}
