/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.overview.stabilization;

import java.util.Hashtable;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.overview.DiffOverview;

public class ScrollingMemorizer extends StabilizationMethod {

	Hashtable<String, Float> translateY = new Hashtable<String, Float>();
	StabilizationMethod method;
	
	public ScrollingMemorizer(StabilizationMethod method) {
		super(method.getOverview(), method.getDetailView());
		this.method = method;
	}
	
	public void documentChanged(DocumentAnimator doc) {
		super.documentChanged(doc);
		method.documentChanged(doc);
		updateScrollingVector();
	}
	
	@Override
	public void setViewOfInterest(boolean validate) {
		translateY.clear();
		method.setViewOfInterest(validate);
		updateScrollingVector();
	}
	
	private void updateScrollingVector() {
		if (document != null) {
			String file1 = document.getFilename1();
			String file2 = document.getFilename2();
			if (!translateY.containsKey(file1))
				translateY.put(file1, method.getStartEndScrollingVector()[0]);
			if (!translateY.containsKey(file2))
				translateY.put(file2, method.getStartEndScrollingVector()[1]);
			scrollingVector[0] = translateY.get(file1);
			scrollingVector[1] = translateY.get(file2);
		} else {
			scrollingVector[0] = 0;
			scrollingVector[1] = 0;
		}
	}
	
	public float[] getStartEndScrollingVector() {
		return scrollingVector;
	}

}
