/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.overview.stabilization;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.overview.DiffOverview;

public abstract class StabilizationMethod {

	protected DiffOverview overview;
	protected DiffAnimator detailview;
	protected DocumentAnimator document = null;
	protected boolean feedbackEnabled = true;
	protected float[] scrollingVector = new float[]{0, 0};

	public StabilizationMethod(DiffOverview overview, DiffAnimator detailview) {
		this.overview = overview;
		this.detailview = detailview;
	}
	
	/**
	 * Called when new revisions have just been loaded and processed. The layout
	 * objects (paragraphs, blocks and words) have changed.
	 */
	public void documentChanged(DocumentAnimator doc) {
		this.document = doc;
	}
	
	/**
	 * Called when the user has just finished scrolling manually (mouse up).
	 * This specifies his view of interest.
	 */
	public abstract void setViewOfInterest(boolean validate);
	
	/**
	 * Called at every animation step.
	 */
	public void stabilize() {
		if (document == null)
			return;
		
		float t = document.getMotionAnimationState();
		float[] scroll = getStartEndScrollingVector();
		float y = scroll[0] + (scroll[1] - scroll[0]) * t;
		overview.moveThumb(overview.doc2bar(y));
	}
	
	/**
	 * Returns the document translation for V1 and V2.
	 */
	public float[] getStartEndScrollingVector() {
		return scrollingVector;
	}
	
	public void setFeedbackEnabled(boolean enabled) {
		feedbackEnabled = enabled;
	}
	
	protected float getLocationWithinBounds(float docY, float t) {
		float thumbY = overview.doc2bar(docY);
		thumbY = overview.getThumbLocationWithinBounds(thumbY, t);
		docY = overview.bar2doc(thumbY);
		return docY;
	}
	
	protected DiffOverview getOverview() {
		return overview;
	}
	
	protected DiffAnimator getDetailView() {
		return detailview;
	}

}
