/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.overview.stabilization;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.TextBlock;
import fr.aviz.diffanimation.overview.DiffOverview;

public class TextAnchorStabilizer extends StabilizationMethod {

	TextBlock anchor = null;
	
	float anchor_offset_y = 0;

	public TextAnchorStabilizer(DiffOverview overview, DiffAnimator detailview) {
		super(overview, detailview);
	}
	
	/**
	 * Called when new revisions have just been loaded and processed. The layout
	 * objects (paragraphs, blocks and words) have changed.
	 */
	@Override
	public void documentChanged(DocumentAnimator doc) {
		super.documentChanged(doc);
		updateAnchor();
	}
	
	/**
	 * Updates the text anchor used for stabilizing text during animation.
	 * Uses the first block fully visible on the detailed view.
	 */
	@Override
	public void setViewOfInterest(boolean validate) {

		if (document == null)
			return;

		// Chose a piece of text to anchor to
		float docy0 = overview.bar2doc(overview.thumbY());
		anchor = document.getTextBlockAt(docy0);
		anchor_offset_y = document.getTextBlockTop(anchor) - docy0;
		
		document.setVisibleAnchor(feedbackEnabled ? anchor : null);
		detailview.repaint();
		
		updateStartEndScrollingVector();
	}
	
	/**
	 * Update the anchor object using the location of the previous anchor
	 */
	private void updateAnchor() {
		
		if (document == null)
			return;
		
		float docy0 = overview.bar2doc(overview.thumbY());
		anchor = document.getTextBlockAt(docy0 + anchor_offset_y);
		anchor_offset_y = document.getTextBlockTop(anchor) - docy0;
		
		document.setVisibleAnchor(feedbackEnabled ? anchor : null);
		detailview.repaint();

		updateStartEndScrollingVector();
	}
	
	/**
	 * Moves the thumb so that the anchor does not move.
	 */
/*	@Override
	public void stabilize() {
	
		if (anchor == null)
			return;
		
		if (document == null)
			return;
		
		float new_anchor_y = document.getTextBlockTop(anchor);
		float dy = new_anchor_y - anchor_y;
		overview.moveThumb(overview.thumbY() + overview.docH2barH(dy));
		anchor_y = new_anchor_y;
		
		// If the anchor is still there but its scale is too small
		// (block got invisible), get a new anchor.
//		if (document.containsTextBlock(anchor) && anchor.getScalex() == TextBlock.MIN_SCALE)
//			updateAnchor();
	}
	*/
	
	public void setFeedbackEnabled(boolean enabled) {
		super.setFeedbackEnabled(enabled);
		if (document == null)
			return;
		document.setVisibleAnchor(feedbackEnabled ? anchor : null);
		detailview.repaint();
	}
	
	/**
	 * Updates the document translation for V1 and V2.
	 */
	protected void updateStartEndScrollingVector() {

		if (anchor == null || document == null) {
			scrollingVector[0] = 0;
			scrollingVector[1] = 0;
			return;
		}
		
		// Get the location of the anchor at V1 and V2
		document.setAnimationCoeff(DocumentAnimator.Coeff.ALT);
		document.setAnimationState(0);
		float y0 = document.getTextBlockTop(anchor);
		document.setAnimationState(1);
		float y1 = document.getTextBlockTop(anchor);
		document.setAnimationCoeff(DocumentAnimator.Coeff.DEFAULT);
		
		// Update the scrolling vector
		scrollingVector[0] = y0 - anchor_offset_y;
		scrollingVector[1] = y1 - anchor_offset_y;
		
		// Take into account thumb range constraints
		scrollingVector[0] = getLocationWithinBounds(scrollingVector[0], 0);
		scrollingVector[1] = getLocationWithinBounds(scrollingVector[1], 1);		
	}
}
