/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.overview.stabilization;

import fr.aviz.diffanimation.DiffAnimator;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.overview.DiffOverview;

public class TopBottomStabilizer extends StabilizationMethod {

	public enum Anchors {
		NONE, DOCUMENT_TOP, DOCUMENT_BOTTOM
	}
	
	Anchors anchor = Anchors.DOCUMENT_TOP;
	
	public TopBottomStabilizer(DiffOverview overview, DiffAnimator detailview) {
		super(overview, detailview);
	}

	public boolean thumbOnTop() {
		return overview.thumbY() - 2 <= overview.barYmin();
	}
	
	public boolean thumbOnBottom() {
		if (document == null)
			return false;
		return overview.bar2doc(overview.thumbY() + overview.thumbHeight() + 2) >= document.getCurrentHeight();
	}
	
	@Override
	public void documentChanged(DocumentAnimator doc) {
		super.documentChanged(doc);
		updateStartEndScrollingVector();
	}
	
	@Override
	public void setViewOfInterest(boolean validate) {
//System.out.println(anchor);
		if (document == null)
			return;
		if (thumbOnTop()) {
			anchor = Anchors.DOCUMENT_TOP;
		} else if (thumbOnBottom()) {
			anchor = Anchors.DOCUMENT_BOTTOM;
		} else {
			anchor = Anchors.NONE;
		}
		updateStartEndScrollingVector();
	}
	
	/**
	 * Updates the document translation for V1 and V2.
	 */
	protected void updateStartEndScrollingVector() {
		
		if (document == null) {
			scrollingVector[0] = 0;
			scrollingVector[1] = 0;
			return;
		}

		if (anchor == Anchors.DOCUMENT_TOP) {
			scrollingVector[0] = 0;
			scrollingVector[1] = 0;
		} else if (anchor == Anchors.DOCUMENT_BOTTOM) {
			float visibleDocHeight = overview.barH2docH(overview.thumbHeight());
			scrollingVector[0] = document.getHeight(0) - visibleDocHeight;
			scrollingVector[1] = document.getHeight(1) - visibleDocHeight;
		} else {
			scrollingVector[0] = overview.bar2doc(overview.thumbY());
			scrollingVector[1] = overview.bar2doc(overview.thumbY());
		}
		
		// Take into account thumb range constraints
		scrollingVector[0] = getLocationWithinBounds(scrollingVector[0], 0);
		scrollingVector[1] = getLocationWithinBounds(scrollingVector[1], 1);
	}

/*	@Override
	public void stabilize() {
		if (document == null)
			return;
		// Stick to the top or the bottom of the document.
		if (location == locations.DOCUMENT_TOP) {
			overview.moveThumb(0);
		} else if (location == locations.DOCUMENT_BOTTOM) {
			overview.moveThumb(overview.doc2bar(document.getCurrentHeight()) - overview.thumbHeight());
		}
	}
	*/
}
