/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.timeline;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;

import fr.aviz.diffanimation.DiffAnimator;

public class CompoundDiffTimeline extends JPanel {

	DiffTimeline timeline1;
	DiffTimeline timeline2;
	DiffTimeline activeTimeline;
	
	public CompoundDiffTimeline(final DiffAnimator detailView) {
		super();
		setBackground(Color.white);
		timeline1 = new DiffTimeline(detailView, 24); // 24
		timeline2 = new DiffTimeline(detailView, 24);
		timeline2.useShortestEditionPath(true);
		timeline2.setLayoutModel(timeline1);
		setActiveTimeline(timeline1);
		setLayout(new BorderLayout());
		add(timeline1, BorderLayout.CENTER);
		add(timeline2, BorderLayout.SOUTH);
	}

	public void newFiles() {
		timeline1.newFiles();
		timeline2.newFiles();
		setActiveTimeline(timeline1);
	}

	public void clear() {
		timeline1.clear();
		timeline2.clear();
		repaint();
	}

	public void setMaxDocumentHeight(float maxDocumentHeight) {
		timeline1.setMaxDocumentHeight(maxDocumentHeight);
		timeline2.setMaxDocumentHeight(maxDocumentHeight);
	}

	public void advance(int i) {
		if (activeTimeline != null)
			activeTimeline.advance(i);
	}

	public void dragCursor(int i) {
		if (activeTimeline != null)
			activeTimeline.dragCursor(i);
	}

	public void grabCursor() {
		if (activeTimeline != null)
			activeTimeline.grabCursor();
	}

	public void hoverCursor(boolean b) {
		if (activeTimeline != null)
			activeTimeline.hoverCursor(b);
	}

	public void releaseCursor() {
		if (activeTimeline != null)
			activeTimeline.releaseCursor();
	}

	public void updateCursorPosition() {
		if (activeTimeline != null)
			activeTimeline.updateCursorPosition();
	}
	
	public void layoutMarks() {
		timeline1.layoutMarks();
		timeline2.layoutMarks();
	}

	public void setActiveTimeline(DiffTimeline timeline) {
		activeTimeline = timeline;
		DiffTimeline nonactiveTimeline = timeline == timeline1 ? timeline2 : timeline1;
		activeTimeline.setActive(true);
		nonactiveTimeline.setActive(false);
		activeTimeline.setAllVisible(true);
		nonactiveTimeline.hideRedundantMarks(activeTimeline);
				
		repaint();
	}
}
