/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.timeline;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.SpeedNormalizer;
import fr.aviz.diffanimation.DocumentAnimator.PaintMode;
import fr.aviz.utils.DocumentFactory;

public class HistoryFlow {

	RevisionMark mark0, mark1;
	Rectangle2D.Float bounds = new Rectangle2D.Float();
	
	// Bitmap drawing settings
	BufferedImage bitmap = null;
	Graphics2D g = null;
	DocumentAnimator document = null;
	int w, h;
	float maxDocumentHeight = 1;
	int current_x = 0;
	
	boolean visible = true;
	
	public HistoryFlow(RevisionMark mark0, RevisionMark mark1) {
		this.mark0 = mark0;
		this.mark1 = mark1;
	}
	
	public void setVisible(boolean visible) {
		this.visible = visible;
	}
	
	public void layout() {
		float x0 = mark0.bounds.x + mark0.bounds.width + DiffTimeline.MARK_MARGIN;
		float x1 = mark1.bounds.x - DiffTimeline.MARK_MARGIN;
		float y0 = mark0.bounds.y;
		float y1 = mark0.bounds.y + mark0.bounds.height;
		bounds.setRect(x0, y0, x1-x0, y1-y0);
	}
	
	public void paint(Graphics2D g) {
		
		if (!visible)
			return;
		
		if (bitmap != null)
			g.drawImage(bitmap, (int)bounds.x, (int)bounds.y, null);
		
		g.setColor(DiffTimeline.BORDER_COLOR);
		g.drawLine((int)(mark0.bounds.x + mark0.bounds.width), 2, (int)mark1.bounds.x, 2);
		g.drawLine((int)(mark0.bounds.x + mark0.bounds.width), h+1, (int)mark1.bounds.x, h+1);
	}
	
	public void newBitmap(float maxDocumentHeight) {
		w = (int)Math.round(bounds.width);
		h = (int)Math.round(bounds.height);
		this.maxDocumentHeight = maxDocumentHeight;
		bitmap = null;
		current_x = 0;
	}
	
	/**
	 * Iterative method for drawing the bitmap. Returns true when finished.
	 * 
	 * @param maxDocumentHeight
	 * @return
	 */
	public boolean drawBitmapIteration() {
		
		if (w < 1 || h < 1)
			return true;

		if (bitmap == null) {
			// init
			bitmap = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
			document = DocumentFactory.createDocument(mark0.filename, mark1.filename);
			g = (Graphics2D)bitmap.getGraphics();
			return false;
		}
		
		float old_motionpause = document.getMotionPause();
		float t;
		float docwidth = document.getDocumentWidth();
		AffineTransform at0;
		//for (float x=0; x<bounds.width; x+=1) {
		t = current_x / (float)w;
		at0 = g.getTransform();
		g.translate(current_x, 0);
		g.scale(1.0/docwidth, h/maxDocumentHeight);
		document.setMotionPause(SpeedNormalizer.NOMINAL_MOTION_PAUSE);
		document.setAnimationCoeff(DocumentAnimator.Coeff.ALT);
		document.setAnimationState(t);
		g.setColor(Color.white);
		g.fillRect(0, 0, (int)docwidth, (int)maxDocumentHeight);
		document.paint(g, PaintMode.HISTORYFLOW, 1, 1f*maxDocumentHeight/h);
		g.setTransform(at0);
		//}
		document.setAnimationCoeff(DocumentAnimator.Coeff.DEFAULT);
		document.setMotionPause(old_motionpause);

		current_x += 1;
		return (current_x == w);
	}

}
