/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.diffanimation.timeline;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import fr.aviz.diffanimation.gui.GUIUtils;

public class RevisionMark {

	private static Stroke borderStroke = new BasicStroke(2);
	private static float smallboundsratio = 0.4f;
	
	String filename;
	Rectangle2D.Float bounds, smallbounds, border, smallborder;
	GeneralPath leftarrow, rightarrow;
	
	boolean visible = true;
	
	public RevisionMark(String filename) {
		this.filename = filename;
		bounds = new Rectangle2D.Float();
		border = new Rectangle2D.Float();
		if (!isCursor()) {
			smallbounds = new Rectangle2D.Float();
			smallborder = new Rectangle2D.Float();
		} else {
			leftarrow = new GeneralPath();
			rightarrow = new GeneralPath();
		}	
	}
	
	public void setVisible(boolean visible) {
		this.visible = visible;
	}
	
	public void setBounds(float x, float y, float w, float h) {
		bounds.setRect(x, y, w, h);
		updateOtherBounds();
	}
	
	public void translate(float dx) {
		bounds.setRect(bounds.x + dx, bounds.y, bounds.width, bounds.height);
		updateOtherBounds();
	}
	
	public void setX(float x) {
		bounds.setRect(x - bounds.width/2, bounds.y, bounds.width, bounds.height);
		updateOtherBounds();
	}
	
	public boolean isCursor() {
		return filename == null;
	}
	
	private void updateOtherBounds() {
		border.setRect(bounds.x-1, bounds.y-1, bounds.width+2, bounds.height+2);
		if (!isCursor()) {
			smallboundsratio = bounds.width / bounds.height;
			smallbounds.setRect(bounds.x, bounds.y + bounds.height*(0.5f-smallboundsratio/2), bounds.width, bounds.height*smallboundsratio);
			smallborder.setRect(smallbounds.x-1, smallbounds.y-1, smallbounds.width+2, smallbounds.height+2);
		} else {
			final int mx = 3;
			final int s = 5;
			float ymid = border.y + border.height/2;
			leftarrow.reset();
			leftarrow.moveTo(border.x - mx - s, ymid);
			leftarrow.lineTo(border.x - mx, ymid - s);
			leftarrow.lineTo(border.x - mx, ymid + s);
			leftarrow.lineTo(border.x - mx - s, ymid);
			rightarrow.reset();
			rightarrow.moveTo(border.x + border.width + mx + s, ymid);
			rightarrow.lineTo(border.x + border.width + mx, ymid - s);
			rightarrow.lineTo(border.x + border.width + mx, ymid + s);
			rightarrow.lineTo(border.x + border.width + mx + s, ymid);
		}
	}
	
	public float getX() {
		return bounds.x + bounds.width/2;
	}
	
	public String getFilename() {
		return filename;
	}
	
	public void paint(Graphics2D g, boolean small) {
		//g.setColor(GUIUtils.multiplyAlpha(DiffTimeline.MARK_COLOR, opacity));
//		g.setStroke(thi);
		
		if (!visible)
			return;
		
		g.setColor(Color.white);
		g.draw(small ? smallborder : border);
		g.setColor(DiffTimeline.MARK_COLOR);
		g.fill(small ? smallbounds : bounds);
	}
	
	public void paintBorder(Graphics2D g, boolean small, Color fillcolor, Color bordercolor) {
		g.setColor(fillcolor);
		g.fill(small ? smallbounds : bounds);
		
		g.setColor(bordercolor);
		g.setStroke(borderStroke);
		g.draw(small ? smallborder : border);
		g.setStroke(DiffTimeline.THIN_STROKE);
	}
	
	public void paintLeftArrow(Graphics2D g, Color fillcolor, Color bordercolor) {
		g.setStroke(borderStroke);
		g.setColor(fillcolor);
		g.fill(leftarrow);
		g.setColor(bordercolor);
		g.draw(leftarrow);
		g.setStroke(DiffTimeline.THIN_STROKE);
	}
	
	public void paintRightArrow(Graphics2D g, Color fillcolor, Color bordercolor) {
		g.setStroke(borderStroke);
		g.setColor(fillcolor);
		g.fill(rightarrow);
		g.setColor(bordercolor);
		g.draw(rightarrow);
		g.setStroke(DiffTimeline.THIN_STROKE);
	}

	public boolean contains(int x, float width) {
		return (x >= bounds.x - width/2 && x <= bounds.x + bounds.width + width/2);
	}	
}
