/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.utils;

import java.awt.Graphics2D;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Hashtable;

import fr.aviz.diffanimation.Document;
import fr.aviz.diffanimation.DocumentAnimator;
import fr.aviz.diffanimation.Settings;

public class DocumentFactory {

	static Hashtable<String, DocumentAnimator> cache = new Hashtable<String, DocumentAnimator>();
	static Settings settings;
	
	public static void setSettings(Settings settings_) {
		settings = settings_;
	}
	
	/**
	 * Support for caching.
	 */
	public static DocumentAnimator createDocument(String filename1, String filename2) {
		String key = filename1 + " -> " + filename2;
		if (cache.containsKey(key))
			return cache.get(key);
		DocumentAnimator newdoc = new DocumentAnimator(filename1, filename2, settings);
		cache.put(key, newdoc);
		return newdoc;
	}
	
	/**
	 * Creates and caches all document animations from version n to version n+1.
	 */
	public static void createDocuments(String[] filenames) {
		for (int i=0; i<filenames.length-1; i++)
			createDocument(filenames[0], filenames[1]);
	}
	
	/**
	 * Empties the cache
	 */
	public static void clearCache() {
		cache.clear();
	}
	
	public static void documentsFontChanged() {
		for (DocumentAnimator doc : cache.values()) {
			doc.markWordSizesDirty();
			doc.markStartEndGeometryDirty();
//			doc.maybeUpdateGeometry();
		}
	}
	
	public static void documentsWidthChanged() {
		for (DocumentAnimator doc : cache.values()) {
			doc.markStartEndGeometryDirty();
//			doc.maybeUpdateGeometry();
		}
	}

}
