/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.utils;

import java.awt.font.TextLayout;
import java.util.Hashtable;

import fr.aviz.diffanimation.Settings;

/**
 * A cache for TextLayout.getAdvance(). This is a very costly method, so this cache mechanism
 * yields impressive gains when reflowing text.
 * 
 * @author dragice
 *
 */
public class WordAdvanceCache {

	static boolean enabled = true;
	
	static Hashtable<String, Float> advances = new java.util.Hashtable<String, Float>();
	static Hashtable<String, Float> visibleAdvances = new java.util.Hashtable<String, Float>();
	static TextLayout currentNominalTextLayout = null;
	
	static String lastWord = null;
	static TextLayout lastTextLayout = null;
	
	public static void enable(boolean enable) {
		enabled = enable;
	}
	
	public static float getAdvance(Settings settings, String word) {
		return getAdvance(false, settings, word);
	}
	
	public static float getVisibleAdvance(Settings settings, String word) {
		return getAdvance(true, settings, word);

	}
	
	private static float getAdvance(boolean visible, Settings settings, String word) {
		if (settings.nominalTextLayout != currentNominalTextLayout) {
			clearCache();
			currentNominalTextLayout = settings.nominalTextLayout;
		}
		Hashtable<String, Float> hashtable = visible ? visibleAdvances : advances;
		if (!enabled || !hashtable.containsKey(word)) {
			TextLayout layout;
			if (word.equals(lastWord))
				layout = lastTextLayout; // if we created the right text layout just before, reuse it.
			else
				layout = new TextLayout(word, Settings.TEXT_FONTS[settings.textFontIndex], settings.fontRenderContext);
			lastWord = word;
			lastTextLayout = layout;
			float advance = visible ? layout.getVisibleAdvance() : layout.getAdvance();
			if (enabled)
				hashtable.put(word, advance);
			return advance;
		} else {
			return hashtable.get(word).floatValue();
		}

	}

	public static void clearCache() {
		advances.clear();
		visibleAdvances.clear();
	}
}
