/* 
 * Copyright (c) 2010 Microsoft Corporation and Inria. 
 * Contributors: Fanny Chevalier and Pierre Dragicevic. 
 * 
 * You may distribute under the terms of either the CeCILL-B License or the 
 * CeCILL version 2 License.
 */
package fr.aviz.wikidiffparser;

import java.util.HashMap;
import java.util.Map.Entry;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.*;


public class ContentHandler extends DefaultHandler {
	private StringBuffer data;
	private int currentElement;	// 0 = ignore
								// 1 = page

	
	// Files-related stats
	private long nbFiles = 0;
	private long insertionInFile = 0;
	private long deletionInFile = 0;
	private long moveInFile = 0;
	
	private long sizeInsertionInFile = 0;
	private long sizeDeletionInFile = 0;
	private long sizeMoveInFile = 0;
	
	private StatCalculator calculatorInsertions;
	private StatCalculator calculatorDeletions;
	private StatCalculator calculatorMoves;
	
	private StatCalculator calculatorSizeInsertions;
	private StatCalculator calculatorSizeDeletions;
	private StatCalculator calculatorSizeMoves;
	
	private long nbInsertions = 0;
	private long nbDeletions = 0;
	private long nbMove = 0;
	
	// globla stats
	private long sizeInsertions = 0;
	private long sizeDeletions = 0;
	private long sizeMove = 0;
	
	
	
	public ContentHandler(){
		super();
		calculatorInsertions = new StatCalculator();
		calculatorDeletions = new StatCalculator();
		calculatorMoves = new StatCalculator();
		
		calculatorSizeInsertions = new StatCalculator();
		calculatorSizeDeletions = new StatCalculator();
		calculatorSizeMoves = new StatCalculator();
	}
	
	
	/*
	    public void setDocumentLocator (Locator locator) { 
		}

	    public void startDocument ()
		throws SAXException  { 
		}

	    public void endDocument()
		throws SAXException {  
		}

	    public void startPrefixMapping (String prefix, String uri)
		throws SAXException {    
		}

	    public void endPrefixMapping (String prefix)
		throws SAXException  {  
		}
*/
	
	    public void startElement (String uri, String localName,
				      String qName, Attributes attributes)
		throws SAXException  {  
	    	currentElement = 0; data = new StringBuffer();
	    	long value = 0;
	    	if (attributes.getLength() != 0){
                for(int index = 0; index < attributes.getLength(); index ++){
                	if (attributes.getLocalName(index) == "length")
                		value = Long.parseLong(attributes.getValue(index));
                }
        	}
	        if (localName.equals("insert")) {
	        	insertionInFile++;
	        	sizeInsertionInFile+=value;
	        }
	        else if (localName.equals("delete")) {
	        	deletionInFile++;
	        	sizeDeletionInFile+=value;
	        }
	        else if (localName.equals("move")) {
	        	moveInFile++;
	        	sizeMoveInFile+=value;
	        }
	        else if (localName.equals("page")) {
	        	currentElement = 1;
	        	insertionInFile = 0;
	        	deletionInFile = 0;
	        	moveInFile = 0;
	        	sizeInsertionInFile = 0;
	        	sizeDeletionInFile = 0;
	        	sizeMoveInFile = 0;
	        }
	    }

	    
	    public void endElement (String uri, String localName,
				    String qName)
		throws SAXException {  
		  if (localName == "page") {
			  nbInsertions += insertionInFile;
			  nbDeletions += deletionInFile;
			  nbMove += moveInFile;
			  sizeInsertions += sizeInsertionInFile;
			  sizeDeletions += sizeDeletionInFile;
			  sizeMove += sizeMoveInFile;
			  
			  calculatorInsertions.addValue(insertionInFile);
			  calculatorDeletions.addValue(deletionInFile);
			  calculatorMoves.addValue(moveInFile);
			  
			  calculatorSizeInsertions.addValue(sizeInsertionInFile);
			  calculatorSizeDeletions.addValue(sizeDeletionInFile);
			  calculatorSizeMoves.addValue(sizeMoveInFile);
		  }
	    }
	    
/*
	    public void characters (char ch[], int start, int length)
		throws SAXException {  
	    	if(currentElement != 0) data.append(ch, start, length); 
		}*/

	    /*
	    public void ignorableWhitespace (char ch[], int start, int length)
		throws SAXException {  
		  System.out.println("Hello from ignorableWhitespace()!"); 
		}

	    public void processingInstruction (String target, String data)
		throws SAXException {  
		  System.out.println("Hello from processingInstruction()!"); 
		}

	    public void skippedEntity (String name)
		throws SAXException {  
		  System.out.println("Hello from skippedEntity()!"); 
		}*/
	    
	    public String overallEditions() {
	    	return new String("Insertions: " + nbInsertions + " ("+ sizeInsertions 
	    						+ ")\nDeletions: " + nbDeletions + " (" + sizeDeletions 
	    						+ ")\nMoves:" + nbMove + " (" + sizeMove + ")");
	    }

	    public String getInsertionStats(){
	    	return getStats(calculatorInsertions);
	    }
	    
	    public String getDeletionStats(){
	    	return getStats(calculatorDeletions);
	    }
	    
	    public String getMoveStats(){
	    	return getStats(calculatorMoves);	    	
	    }
	    
	    
	    public String getSizeInsertionStats(){
	    	return getStats(calculatorSizeInsertions);	    	
	    }
	    
	    public String getSizeDeletionStats(){
	    	return getStats(calculatorSizeDeletions);
	    }
	    
	    public String getSizeMoveStats(){
	    	return getStats(calculatorSizeMoves);
	    }
	    
	    
	    public String getInsertionDistribution(){
	    	return getDistribution(calculatorInsertions);
	    }
	    
	    public String getDeletionDistribution(){
	    	return getDistribution(calculatorDeletions);
	    }
	    
	    public String getMoveDistribution(){
	    	return getDistribution(calculatorMoves);	    	
	    }
	    
	    
	    public String getSizeInsertionDistribution(){
	    	return getDistribution(calculatorSizeInsertions);	    	
	    }
	    
	    public String getSizeDeletionDistribution(){
	    	return getDistribution(calculatorSizeDeletions);
	    }
	    
	    public String getSizeMoveDistribution(){
	    	return getDistribution(calculatorSizeMoves);
	    }
	    
	    private String getDistribution(StatCalculator calc){
	    	String  str = new String();
	    	HashMap <Long, Integer> map= calc.getDistribution();
	    	for(Entry<Long, Integer> entry : map.entrySet()) {
	    	    str+=(entry.getKey() + " " + entry.getValue() + '\n');
	    	}
	    	return str;
	    }

	    private String getStats(StatCalculator calc){
	    	String str = new String("Min: " + calc.getMin() + '\n' +
	    			"Max: " + calc.getMax() + '\n' +
	    			"Mean: " + calc.getMean() + '\n' +
	    			"Dev: " + calc.getStandardDeviation() + '\n' +
	    			"Med: " + calc.getMedian() + '\n');
	    	return str;
	    }

}
